package com.circuitengine.model.Components;

import java.awt.Graphics;

import com.circuitengine.model.TiePoint;

/*AUTHORS:

- Kevin Stueve (2009-12-20): initial published version
#*****************************************************************************
#       Copyright (C) 2009 Kevin Stueve kstueve@uw.edu
#
#  Distributed under the terms of the GNU General Public License (GPL)
#                  http://www.gnu.org/licenses/
#*****************************************************************************
*/
public final class ACCurrentSource extends ElectricComponent {
  double current;
  double frequency;
  public ACCurrentSource(int x, int y){
    super(x,y);
    init();
  }
  public void init(){
    primary="Current";
    primaryUnit="Amp";
    primaryUnitPlural="Amps";
    primarySymbol="I";
    primaryUnitSymbol="A";
    preselectedPrimaryPrefix="mA";
    preselectedFrequencyPrefix="Hz";
    frequencySymbol="f";
    preselectedPhaseShiftPrefix="s";
  }
  public void draw(Graphics g) {
    super.draw(g);
    int x1=head.getX();
    int x2=tail.getX();
    int y1=head.getY();
    int y2=tail.getY();
    int rise = y2 - y1;
    int run = x2 - x1;
    int currentSourceSize=11;
    double radius;
    double distance=Math.sqrt(rise*rise+run*run);
      double xm=(x1+x2)/2;
      double ym=(y1+y2)/2;
      if (distance>=2*currentSourceSize){
          g.drawLine(x1,y1,(int)(x1+run/2.0-currentSourceSize*run/distance),(int)(y1+rise/2-currentSourceSize*rise/distance));
          g.drawLine((int)(x2-run/2.0+currentSourceSize*run/distance),(int)(y2-rise/2.0+currentSourceSize*rise/distance),x2,y2);
          g.drawOval((int)(xm-currentSourceSize),(int)(ym-currentSourceSize),
                     2*currentSourceSize,2*currentSourceSize);
          g.drawArc((int)Math.round(xm-currentSourceSize),(int)Math.round(ym-currentSourceSize/2.0), 
                    currentSourceSize,currentSourceSize, 180, -180);
          g.drawArc((int)Math.round(xm),(int)Math.round(ym-currentSourceSize/2.0), 
                    currentSourceSize,currentSourceSize, 180, 180);
          radius=currentSourceSize;
          //g.drawString("+",(int)(x1+run/2.0-(METER_SIZE+10)*run/distance-10*rise/distance),(int)(y1+rise/2-(METER_SIZE+10)*rise/distance+10*run/distance));
      }else{
          radius=distance/2;
          g.drawOval((int)(xm-distance/2),(int)(ym-distance/2),(int)distance,(int)distance);
          g.drawArc((int)Math.round(xm-distance/2),(int)Math.round(ym-distance/4.0), 
                    (int)Math.round(distance/2),(int)Math.round(distance/2), 180, -180);
          g.drawArc((int)Math.round(xm),(int)Math.round(ym-distance/4.0), 
                    (int)Math.round(distance/2),(int)Math.round(distance/2), 180, 180);
      }
      if (distance>0){
        double a=7.0;
        double b=4.0;
              g.drawLine((int)(xm+a/currentSourceSize*radius*run/distance),
                 (int)(ym+a/currentSourceSize*radius*rise/distance),
                 (int)(xm-a/currentSourceSize*radius*run/distance),
                 (int)(ym-a/currentSourceSize*radius*rise/distance));
              g.drawLine((int)(xm+a/currentSourceSize*radius*run/distance),
                         (int)(ym+a/currentSourceSize*radius*rise/distance),
                         (int)(xm+b/currentSourceSize*radius*run/distance+b/currentSourceSize*radius*rise/distance),
                         (int)(ym+b/currentSourceSize*radius*rise/distance-b/currentSourceSize*radius*run/distance));
              g.drawLine((int)(xm+a/currentSourceSize*radius*run/distance),
                         (int)(ym+a/currentSourceSize*radius*rise/distance),
                         (int)(xm+b/currentSourceSize*radius*run/distance-b/currentSourceSize*radius*rise/distance),
                         (int)(ym+b/currentSourceSize*radius*rise/distance+b/currentSourceSize*radius*run/distance));
      
      }
  }
}
