package com.circuitengine.model.Components;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.RenderingHints;
import java.text.DecimalFormat;

import com.circuitengine.model.TiePoint;
import com.circuitengine.model.LinearAlgebra.Rational;
import com.circuitengine.view.GraphFrame;
/*AUTHORS:

- Kevin Stueve (2009-12-20): initial published version
#*****************************************************************************
#       Copyright (C) 2009 Kevin Stueve kstueve@uw.edu
#
#  Distributed under the terms of the GNU General Public License (GPL)
#                  http://www.gnu.org/licenses/
#*****************************************************************************
*/
abstract public class ElectricComponent {
  public Color color;
  public TiePoint head;
  public TiePoint tail;
  private static String micro="\u03BC";
  private static String[] siprefixes={"Y","Z","E","P","T","G","M","k","","m",micro,"n","p","f","a","z","y"};
  public String primary;
  public String secondary;
  public String primaryUnit;
  public String primaryUnitPlural;
  public String primarySymbol;
  public String primaryUnitSymbol;
  public String secondaryUnitSymbol;
  public String preselectedPrimaryPrefix;
  public String preselectedFrequencyPrefix;
  public String preselectedPhaseShiftPrefix;
  public String enteredPrimaryValue;
  public String selectedPrimaryPrefix;
  public String selectedFrequencyPrefix;
  public String selectedPhaseShiftPrefix;
  public String frequencySymbol;
  public Double primaryValue;//changed to double from Double on Dec 29 2008
  public Double frequency;
  public Double phaseShift;
  public Double calculatedValue;
  public String enteredFrequency;
  public String enteredPhaseShift;
  public double secondaryValue=0.0;
  public boolean isConstant,isHeldConstant;
  public String fullValue="";
  public GraphFrame gf;
  //static{
  //  System.out.println(formatValue(1));
  //  System.out.println(formatValue(10));
  //  System.out.println(formatValue(0.99999999));
  //  System.out.println(formatValue(9.999999999999999));
  //}
  public double y1,yj,zj,k1,k2,k3,k4,k5,k6,I1,y2,y3,originalValue,twoStepValue,oneStepValue,originalCurrent,delta1;
  
  //import java.text.DecimalFormat;
  public static String formatValue(double d){
    if (Double.isInfinite(d)||Double.isNaN(d)){
      return "Error ";
    }
    String sign=" ";
    if (d==0.0){
     return "0.00 " ;
    }
    if (d<0){
      sign="-" ;
    }
    double absd=Math.abs(d);
    int p=(int)Math.floor(Math.log(absd)/Math.log(1000));
    double mantissa=absd/Math.pow(1000,p);
    //System.out.println(mantissa+" "+p*3);
    //System.out.println((int)(-3.5));
    DecimalFormat format;
    if (mantissa<10){
      format=new DecimalFormat("0.00");
      //mantissa=((int)(mantissa*100))/100; 
    }else if (mantissa<100){
      format=new DecimalFormat("00.0");
      //mantissa=((int)(mantissa*10))/10;
    }else{
     format=new DecimalFormat("000");
    }
    if (absd<10e-24||absd>=10e27|| 8-p<0 ||8-p>=siprefixes.length){
      return sign+format.format(mantissa)+"E"+3*p+"";
    }
    //int i=8-p;
    
    return sign+format.format(mantissa)+siprefixes[8-p]+"";
  }
  
  public ElectricComponent(final int x, final int y){
    head = new TiePoint(x,y);
    tail = new TiePoint(x,y);
    connectHeadAndTail();
  }
  public void moveEnd(final int x, final int y) {
    tail.setX(x);
    tail.setY(y);
    connectHeadAndTail();
  }
  public void setTail(TiePoint theTiePoint){
    disconnectHeadAndTail();
    tail = theTiePoint;
    connectHeadAndTail();
  }
  public void setHead(TiePoint theTiePoint) {
    disconnectHeadAndTail();
    head = theTiePoint;
    connectHeadAndTail();
  }
  public void connectHeadAndTail(){
    head.myConnectedTo.add(tail);
    tail.myConnectedTo.add(head);
    head.in.add(this);
    tail.out.add(this);
  }
  public void disconnectHeadAndTail(){
    head.myConnectedTo.remove(tail);
    tail.myConnectedTo.remove(head);
    head.in.remove(this);
    tail.out.remove(this);
  }
  public boolean badSize(){
    return head.getX()==tail.getX() && head.getY()==tail.getY();
  }
   public void draw(Graphics g){
     Graphics2D g2d = (Graphics2D) g;
     
     label(g);
     
     g2d.setRenderingHint
     (RenderingHints.KEY_ANTIALIASING,
      RenderingHints.VALUE_ANTIALIAS_ON);
   }
   public double sqr(double x){
     return x*x;
   }
   public void label(Graphics g){
     String unit;
     if (this.primaryValue!=null &&this.primaryValue==1){
         unit=this.primaryUnitSymbol;//primaryUnit;
     }else{
         unit=this.primaryUnitSymbol;//Plural;
     }
     int x1=(int)this.head.getX();
     int y1=(int)this.head.getY();
     int x2=(int)this.tail.getX();
     int y2=(int)this.tail.getY();
     int run=x2-x1;
     int rise=y2-y1;
     double d=Math.sqrt(sqr(run/2.0)+sqr(rise/2.0));
     
  if (this.primaryValue!=null){
      
         if (rise*run>0){
             g.drawString(formatValue(this.primaryValue)+unit,
             (int)(x1+run/2.0+Math.abs(rise)/(2.0*d)*30),
             (int)(y1+rise/2.0-Math.abs(run)/(2.0*d)*30)
             );
         }else{
             //d+=50;
             g.drawString(formatValue(this.primaryValue)+unit,(int)(x1+run/2.0+Math.abs(rise)/2.0/d*30),
                          (int)(y1+rise/2.0+Math.abs(run)/2.0/d*30));
         }
     }
   }
   public void reverse(){
     disconnectHeadAndTail();
     TiePoint oldHead=head;
     this.head=this.tail;
     this.tail=oldHead;
     connectHeadAndTail();
   }
}
