package com.circuitengine.view;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Point;
import java.util.HashSet;
import java.util.Iterator;

import javax.swing.JPanel;

import com.circuitengine.controller.CircuitEngineMouseListener;
import com.circuitengine.model.TiePoint;
import com.circuitengine.model.Components.ACCurrentSource;
import com.circuitengine.model.Components.ACVoltageSource;
import com.circuitengine.model.Components.Ammeter;
import com.circuitengine.model.Components.Battery;
import com.circuitengine.model.Components.CapacitanceMeter;
import com.circuitengine.model.Components.Capacitor;
import com.circuitengine.model.Components.CurrentIntegrator;
import com.circuitengine.model.Components.ElectricComponent;
import com.circuitengine.model.Components.InductanceMeter;
import com.circuitengine.model.Components.Inductor;
import com.circuitengine.model.Components.Meter;
import com.circuitengine.model.Components.Ohmmeter;
import com.circuitengine.model.Components.VoltageIntegrator;
import com.circuitengine.model.Components.Voltmeter;
import com.circuitengine.model.Components.Wire;
import com.circuitengine.model.LinearAlgebra.SparseMatrix;

/*AUTHORS:

- Kevin Stueve (2009-12-20): initial published version
#*****************************************************************************
#       Copyright (C) 2009 Kevin Stueve kstueve@uw.edu
#
#  Distributed under the terms of the GNU General Public License (GPL)
#                  http://www.gnu.org/licenses/
#*****************************************************************************
*/
public final class DrawingPanel extends JPanel {
  CircuitEnginePanel myParentCircuitEnginePanel;
  CircuitEngineMouseListener myCircuitEngineMouseListener;
  SparseMatrix sparseMatrix;
  public boolean updated;
  public double t;
  public double stepSize;

  public DrawingPanel(final CircuitEnginePanel parentCircuitEnginePanel) {
    super();
    myParentCircuitEnginePanel = parentCircuitEnginePanel;
    myCircuitEngineMouseListener =
        new CircuitEngineMouseListener(parentCircuitEnginePanel);
    this.addMouseMotionListener(myCircuitEngineMouseListener);
    this.addMouseListener(myCircuitEngineMouseListener);
    this.setBackground(Color.WHITE);
  }

  public void updateCircuit(){
    //System.out.println("entered update");
    myParentCircuitEnginePanel.isBusy=true;
    if (myParentCircuitEnginePanel.myGridPoints.size()>0){
      sparseMatrix=new SparseMatrix(myParentCircuitEnginePanel,
                                    myParentCircuitEnginePanel.myGridPoints.values().size()+
                                    3*myParentCircuitEnginePanel.myCircuit.electricComponents.size()//max 3 equations
                                    ,
                                    2*myParentCircuitEnginePanel.myGridPoints.values().size()+//v and v'
                                    3*myParentCircuitEnginePanel.myCircuit.electricComponents.size()+
                                    1);
      
      sparseMatrix.setupMatrix2(t);
      //System.out.println("orig");
      //SparseMatrix.printMatrix(sparseMatrix);
      sparseMatrix.refMatrix(sparseMatrix);
      //System.out.println("after ref");
      //SparseMatrix.printMatrix(sparseMatrix);
      sparseMatrix.rrefMatrix(sparseMatrix);
      //System.out.println("after rref");
      //SparseMatrix.printMatrix(sparseMatrix);
      for (ElectricComponent ec:myParentCircuitEnginePanel.myCircuit.electricComponents){
        if (ec.getClass()==Capacitor.class){
          if (ec.primaryValue!=null){
            //System.out.println("Old charge: "+ec.secondaryValue);
            ec.secondaryValue+=sparseMatrix.getDeltaQ(ec);
            //System.out.println("Q= "+ec.secondaryValue);
            //System.out.println("deltaQ= "+sparseMatrix.getDeltaQ(ec));
            //System.out.println("New charge: "+ec.secondaryValue);
            //System.out.println(ec.secondaryValue);
            //System.out.println("changed "+sparseMatrix.getDeltaQ(ec));
          }
        }
        if (ec.getClass()==CurrentIntegrator.class){
          ec.secondaryValue+=sparseMatrix.getDeltaQ(ec);
        }
      }
      
      
      
      sparseMatrix=new SparseMatrix(myParentCircuitEnginePanel,
                                    myParentCircuitEnginePanel.myGridPoints.values().size()+
                                    3*myParentCircuitEnginePanel.myCircuit.electricComponents.size()//max 3 equations
                                    ,
                                    2*myParentCircuitEnginePanel.myGridPoints.values().size()+//v and v'
                                    3*myParentCircuitEnginePanel.myCircuit.electricComponents.size()+
                                    1);
      
      sparseMatrix.setupMatrix3(t);
      //System.out.println("orig");
      //SparseMatrix.printMatrix(sparseMatrix);
      sparseMatrix.refMatrix(sparseMatrix);
      //System.out.println("after ref");
      //SparseMatrix.printMatrix(sparseMatrix);
      sparseMatrix.rrefMatrix(sparseMatrix);
      //System.out.println("after rref");
      //SparseMatrix.printMatrix(sparseMatrix);
      //System.out.println("start");
      for (ElectricComponent ec:myParentCircuitEnginePanel.myCircuit.electricComponents){
        if (ec.getClass()==Inductor.class){
          if (ec.primaryValue!=null){
            //System.out.println("Old charge: "+ec.secondaryValue);
            ec.secondaryValue+=ec.primaryValue*sparseMatrix.getDeltaCurrent(ec);
            //if (sparseMatrix.getCurrent(ec)==0){
            //  ec.secondaryValue=0;
            //}
            //System.out.println("I= "+ec.secondaryValue);
            //System.out.println("deltacurrent= "+sparseMatrix.getDeltaCurrent(ec));
            //System.out.println("New charge: "+ec.secondaryValue);
            //System.out.println(ec.secondaryValue);
            //System.out.println("changed "+sparseMatrix.getDeltaQ(ec));
          }
        }
        else if (ec.getClass()==VoltageIntegrator.class){
          ec.secondaryValue+=sparseMatrix.getDeltaFlux(ec);
        }
      }
      
      sparseMatrix=new SparseMatrix(myParentCircuitEnginePanel,
                                    myParentCircuitEnginePanel.myGridPoints.values().size()+
                                    3*myParentCircuitEnginePanel.myCircuit.electricComponents.size()//max 3 equations
                                    ,
                                    2*myParentCircuitEnginePanel.myGridPoints.values().size()+//v and v'
                                    3*myParentCircuitEnginePanel.myCircuit.electricComponents.size()+
                                    1);
      sparseMatrix.setupMatrix(t);
      sparseMatrix.refMatrix(sparseMatrix);
      sparseMatrix.rrefMatrix(sparseMatrix);
      updateGraphsAndMeters();
      
      //sparseMatrix.setupMatrix();
      double tstep=0.05;
      double finalt=t+tstep;
      long start = System.currentTimeMillis();
      //findConstantCaps();  #commented out Dec 21 08
      if (myParentCircuitEnginePanel.timeOption){
        do{
          //System.out.println("entering rk");
          rk(tstep,tstep);
        }while(t<finalt&&System.currentTimeMillis()<start+0.03);
        updateGraphsAndMeters();
      }
      
      //while (t<finalt&&System.currentTimeMillis()<start+0.5){
        //rk(0.05/2);
       // }
      //System.out.println("end rk"+System.currentTimeMillis());
      
      //below is just for testing
     /* sparseMatrix=new SparseMatrix(myParentCircuitEnginePanel,
                                    myParentCircuitEnginePanel.myGridPoints.values().size()+
                                    3*myParentCircuitEnginePanel.myCircuit.electricComponents.size()//max 3 equations
                                    ,
                                    2*myParentCircuitEnginePanel.myGridPoints.values().size()+//v and v'
                                    3*myParentCircuitEnginePanel.myCircuit.electricComponents.size()+
                                    1);
      sparseMatrix.setupMatrix();
      //System.out.println("original matrix:");
      //SparseMatrix.printMatrix(sparseMatrix);
      
      sparseMatrix.refMatrix(sparseMatrix);
      //System.out.println("after ref:");
      //SparseMatrix.printMatrix(sparseMatrix);
      //System.out.println("after ref");
      
      sparseMatrix.rrefMatrix(sparseMatrix);*/
    }
    
    myParentCircuitEnginePanel.isBusy=false;
    //System.out.println("leaving update");
  }
  public HashSet<ElectricComponent> union(HashSet<ElectricComponent> a,HashSet<ElectricComponent> b){
    HashSet<ElectricComponent> u=new HashSet<ElectricComponent>(a);
    u.addAll(b);
    return u;
  }
  public void findConstantCaps(){
    for (ElectricComponent ec: myParentCircuitEnginePanel.myCircuit.electricComponents){
      if (ec.getClass()==Capacitor.class){
        ec.isConstant=false;
        ec.isHeldConstant=false;
        HashSet<ElectricComponent> seen =new HashSet<ElectricComponent>();
        for (ElectricComponent e:union(ec.head.in,ec.head.out)){
          if ((e.getClass() ==Wire.class ||e.getClass()==Battery.class||e.getClass()==Ammeter.class
              ||ec.getClass()==CurrentIntegrator.class)){
            seen.add(e);
            expand(ec,e,seen);
          }
        }
        //(ec,ec,seen);
      }else{
        ec.isConstant=false;
        ec.isHeldConstant=false;
      }
    }
  }
  private void updateGraphsAndMeters(){
    for (ElectricComponent ec: myParentCircuitEnginePanel.myCircuit.electricComponents){
      if (ec.getClass()==Voltmeter.class){
        Voltmeter v=(Voltmeter)ec;
        v.voltageString=sparseMatrix.getVoltageString(ec,true);
        v.fullValue=sparseMatrix.getVoltageString(ec,false);
        v.calculatedValue=sparseMatrix.getVoltage(ec);
        if (ec.gf!=null&&ec.gf.pb!=null){
          ec.gf.pb.addPoint(0, t, sparseMatrix.getVoltage(ec), true);
          ec.gf.pb.fillPlot();
        }
      }
      if (ec.getClass()==Ammeter.class){
        Ammeter a=(Ammeter)ec;
        a.ampString=sparseMatrix.getCurrentString(ec,true);
        a.fullValue=sparseMatrix.getCurrentString(ec,false);
        a.calculatedValue=sparseMatrix.getCurrent(ec);
        
        if (ec.gf!=null&&ec.gf.pb!=null){
          ec.gf.pb.addPoint(0, t, sparseMatrix.getCurrent(ec), true);
          ec.gf.pb.fillPlot();
        }
      }
      if (ec.getClass()==CurrentIntegrator.class){
        CurrentIntegrator a=(CurrentIntegrator)ec;
        a.chargeString=ElectricComponent.formatValue(a.secondaryValue)+"C";
        a.fullValue=a.secondaryValue+"C";
        a.calculatedValue=a.secondaryValue;
        if (ec.gf!=null&&ec.gf.pb!=null){
          ec.gf.pb.addPoint(0, t, ec.secondaryValue, true);
          ec.gf.pb.fillPlot();
        }
      }
      if (ec.getClass()==VoltageIntegrator.class){
        VoltageIntegrator a=(VoltageIntegrator)ec;
        a.magneticFluxString=ElectricComponent.formatValue(a.secondaryValue)+"Wb";
        a.fullValue=a.secondaryValue+"Wb";
        a.calculatedValue=a.secondaryValue;
        if (ec.gf!=null&&ec.gf.pb!=null){
          ec.gf.pb.addPoint(0, t, ec.secondaryValue, true);
          ec.gf.pb.fillPlot();
        }
      }
      if (ec.getClass()==Ohmmeter.class){
        Ohmmeter a=(Ohmmeter)ec;
        a.ohmString=sparseMatrix.getResistanceString(ec,true);
        a.fullValue=sparseMatrix.getResistanceString(ec,false);
        a.calculatedValue=sparseMatrix.getResistance(ec);
        if (ec.gf!=null&&ec.gf.pb!=null){
          ec.gf.pb.addPoint(0, t, sparseMatrix.getResistance(ec), true);
          ec.gf.pb.fillPlot();
        }
      }
      if (ec.getClass()==CapacitanceMeter.class){
        CapacitanceMeter a=(CapacitanceMeter)ec;
        a.capacitanceString=sparseMatrix.getCapacitanceString(ec,true);
        a.fullValue=sparseMatrix.getCapacitanceString(ec,false);
        a.calculatedValue=sparseMatrix.getCapacitance(ec);
        if (ec.gf!=null&&ec.gf.pb!=null){
          ec.gf.pb.addPoint(0, t, sparseMatrix.getCapacitance(ec), true);
          ec.gf.pb.fillPlot();
        }
      }
      if (ec.getClass()==InductanceMeter.class){
        InductanceMeter a=(InductanceMeter)ec;
        a.inductanceString=sparseMatrix.getInductanceString(ec,true);
        a.fullValue=sparseMatrix.getInductanceString(ec,false);
        a.calculatedValue=sparseMatrix.getInductance(ec);
        if (ec.gf!=null&&ec.gf.pb!=null){
          ec.gf.pb.addPoint(0, t, sparseMatrix.getInductance(ec), true);
          ec.gf.pb.fillPlot();
        }
      }
    }
    for (PhasePlane phasePlane:myParentCircuitEnginePanel.phasePlanes){
      if (phasePlane.xComponent==null||phasePlane.yComponent==null
          ||!myParentCircuitEnginePanel.myCircuit.electricComponents.contains(phasePlane.xComponent)
          ||!myParentCircuitEnginePanel.myCircuit.electricComponents.contains(phasePlane.yComponent)){
        continue;
      }
      phasePlane.pb.addPoint(0, phasePlane.xComponent.calculatedValue,
                             phasePlane.yComponent.calculatedValue, true);
      phasePlane.pb.fillPlot();
    }
  }
  public void expand(ElectricComponent e1,ElectricComponent e2, HashSet<ElectricComponent> seen){
    if (e1.isConstant){
      return;
    }
    for (ElectricComponent ec:union(union(e2.head.in,e2.head.out),union(e2.tail.in,e2.tail.out))){
      if ((ec.getClass() ==Wire.class ||ec.getClass()==Battery.class||ec.getClass()==Ammeter.class
          ||ec.getClass()==CurrentIntegrator.class)
          &&!seen.contains(ec)){
        if (ec.head==e1.tail||ec.tail==e1.tail){
          e1.isConstant=true;
          e1.isHeldConstant=true;
          System.out.println("is constant");
          return;
        }
        seen.add(ec);
        expand(e1,ec,seen);
      }
    }
  }
  
  public void rk2(double h,boolean reset,double t){
    sparseMatrix=new SparseMatrix(myParentCircuitEnginePanel,
                                  myParentCircuitEnginePanel.myGridPoints.values().size()+
                                  3*myParentCircuitEnginePanel.myCircuit.electricComponents.size()//max 3 equations
                                  ,
                                  2*myParentCircuitEnginePanel.myGridPoints.values().size()+//v and v'
                                  3*myParentCircuitEnginePanel.myCircuit.electricComponents.size()+
                                  1);
    sparseMatrix.setupMatrix(t);
    sparseMatrix.refMatrix(sparseMatrix);
    sparseMatrix.rrefMatrix(sparseMatrix);
    for (ElectricComponent ec:myParentCircuitEnginePanel.myCircuit.electricComponents){
      if ((ec.getClass()!=Capacitor.class&&ec.getClass()!=Inductor.class) ||ec.primaryValue==null||ec.isConstant){
        if (ec.getClass()!=CurrentIntegrator.class&&ec.getClass()!=VoltageIntegrator.class
            &&ec.getClass()!=ACVoltageSource.class&&ec.getClass()!=ACCurrentSource.class){
          continue;
        }
      }
      if (ec.getClass()==Capacitor.class||ec.getClass()==CurrentIntegrator.class
          ||ec.getClass()==ACCurrentSource.class){
         ec.k1=sparseMatrix.getCurrent(ec);
         //System.out.println("k1:"+ec.k2);
      }else if (ec.getClass()==Inductor.class){
        ec.k1=ec.primaryValue*sparseMatrix.getCurrentPrime(ec);
      }else {//voltage integrator or ACVoltageSource
        ec.k1=sparseMatrix.getVoltage(ec);
      }
      ec.I1=ec.k1;
      ec.y1=ec.secondaryValue;
      ec.secondaryValue=ec.y1+h/2*ec.k1;
    }
    sparseMatrix.rows.clear();
    sparseMatrix.cols.clear();
    sparseMatrix.setupMatrix(t+h/2);//added dec 21 08
    sparseMatrix.refMatrix(sparseMatrix);
    sparseMatrix.rrefMatrix(sparseMatrix);
    for (ElectricComponent ec:myParentCircuitEnginePanel.myCircuit.electricComponents){
      if ((ec.getClass()!=Capacitor.class&&ec.getClass()!=Inductor.class)||ec.primaryValue==null||ec.isConstant){
        if (ec.getClass()!=CurrentIntegrator.class&&ec.getClass()!=VoltageIntegrator.class
            &&ec.getClass()!=ACVoltageSource.class&&ec.getClass()!=ACCurrentSource.class){
          continue;
        }
      }
      if (ec.getClass()==Capacitor.class||ec.getClass()==CurrentIntegrator.class
          ||ec.getClass()==ACCurrentSource.class){
        ec.k2=sparseMatrix.getCurrent(ec);
        //System.out.println("k2:"+ec.k2);
      }else if (ec.getClass()==Inductor.class){
       ec.k2=ec.primaryValue*sparseMatrix.getCurrentPrime(ec);
      }else{//voltage integrator or ACVoltageSource
        ec.k2=sparseMatrix.getVoltage(ec);
      }
      ec.secondaryValue=ec.y1+h/2*ec.k2;
    }
    sparseMatrix.rows.clear();
    sparseMatrix.cols.clear();
    sparseMatrix.setupMatrix(t+h/2);//added dec 21 08
    sparseMatrix.refMatrix(sparseMatrix);
    sparseMatrix.rrefMatrix(sparseMatrix);
    for (ElectricComponent ec:myParentCircuitEnginePanel.myCircuit.electricComponents){
      if ((ec.getClass()!=Capacitor.class&&ec.getClass()!=Inductor.class)||ec.primaryValue==null||ec.isConstant){
        if (ec.getClass()!=CurrentIntegrator.class&&ec.getClass()!=VoltageIntegrator.class
            &&ec.getClass()!=ACVoltageSource.class&&ec.getClass()!=ACCurrentSource.class){
          continue;
        }
      }
      if (ec.getClass()==Capacitor.class||ec.getClass()==CurrentIntegrator.class
          ||ec.getClass()==ACCurrentSource.class){
        ec.k3=sparseMatrix.getCurrent(ec);
        //System.out.println("k3:"+ec.k3);
      }else if (ec.getClass()==Inductor.class){
       ec.k3=ec.primaryValue*sparseMatrix.getCurrentPrime(ec);
      }else{//voltage integrator or ACVoltageSource
        ec.k3=sparseMatrix.getVoltage(ec);
      }
      ec.secondaryValue=ec.y1+h*ec.k3;
    }
    sparseMatrix.rows.clear();
    sparseMatrix.cols.clear();
    sparseMatrix.setupMatrix(t+h);//added dec 21 08
    sparseMatrix.refMatrix(sparseMatrix);
    sparseMatrix.rrefMatrix(sparseMatrix);
    for (ElectricComponent ec:myParentCircuitEnginePanel.myCircuit.electricComponents){
      if ((ec.getClass()!=Capacitor.class&&ec.getClass()!=Inductor.class)||ec.primaryValue==null||ec.isConstant){
        if (ec.getClass()!=CurrentIntegrator.class&&ec.getClass()!=VoltageIntegrator.class
            &&ec.getClass()!=ACVoltageSource.class&&ec.getClass()!=ACCurrentSource.class){
          continue;
        }
      }
      if (ec.getClass()==Capacitor.class||ec.getClass()==CurrentIntegrator.class
          ||ec.getClass()==ACCurrentSource.class){
        ec.k4=sparseMatrix.getCurrent(ec);
        //System.out.println("k4:"+ec.k4);
      }else if (ec.getClass()==Inductor.class){
       ec.k4=ec.primaryValue*sparseMatrix.getCurrentPrime(ec);
      }else{//voltage integrator or ACVoltageSource
        ec.k4=sparseMatrix.getVoltage(ec);
      }
      ec.y2=ec.y1+h*(ec.k1+2*ec.k2+2*ec.k3+ec.k4)/6.0;
      //ec.secondaryValue=ec.y2;
      if (reset){
        ec.secondaryValue=ec.y1;
      }
    }
  }
  public void rk(double h,double origh){
    //System.out.println(h);
    for (ElectricComponent ec:myParentCircuitEnginePanel.myCircuit.electricComponents){
      if ((ec.getClass()!=Capacitor.class&&ec.getClass()!=Inductor.class)||ec.primaryValue==null||ec.isConstant){
        if (ec.getClass()!=CurrentIntegrator.class&&ec.getClass()!=VoltageIntegrator.class
            &&ec.getClass()!=ACVoltageSource.class&&ec.getClass()!=ACCurrentSource.class){
          continue;
        }
      }
      ec.originalValue=ec.secondaryValue;
    }
    rk2(h/2,false,t);
    for (ElectricComponent ec:myParentCircuitEnginePanel.myCircuit.electricComponents){
      if ((ec.getClass()!=Capacitor.class&&ec.getClass()!=Inductor.class)||ec.primaryValue==null||ec.isConstant){
        if (ec.getClass()!=CurrentIntegrator.class&&ec.getClass()!=VoltageIntegrator.class
            &&ec.getClass()!=ACVoltageSource.class&&ec.getClass()!=ACCurrentSource.class){
          continue;
        }
      }
      ec.originalCurrent=ec.I1;
    }
    rk2(h/2,false,t+h/2);
    for (ElectricComponent ec:myParentCircuitEnginePanel.myCircuit.electricComponents){
      if ((ec.getClass()!=Capacitor.class&&ec.getClass()!=Inductor.class)||ec.primaryValue==null||ec.isConstant){
        if (ec.getClass()!=CurrentIntegrator.class&&ec.getClass()!=VoltageIntegrator.class
            &&ec.getClass()!=ACVoltageSource.class&&ec.getClass()!=ACCurrentSource.class){
          continue;
        }
      }
      ec.twoStepValue=ec.y2;
      ec.secondaryValue=ec.originalValue;
    }
    rk2(h,false,t);
    for (ElectricComponent ec:myParentCircuitEnginePanel.myCircuit.electricComponents){
      if ((ec.getClass()!=Capacitor.class&&ec.getClass()!=Inductor.class)||ec.primaryValue==null||ec.isConstant){
        if (ec.getClass()!=CurrentIntegrator.class&&ec.getClass()!=VoltageIntegrator.class
            &&ec.getClass()!=ACVoltageSource.class&&ec.getClass()!=ACCurrentSource.class){
          continue;
        }
      }
      ec.oneStepValue=ec.y2;
      ec.secondaryValue=ec.originalValue;
    }
    double minh=h;
    for (ElectricComponent ec:myParentCircuitEnginePanel.myCircuit.electricComponents){
      if ((ec.getClass()!=Capacitor.class&&ec.getClass()!=Inductor.class)||ec.primaryValue==null||ec.isConstant){
        if (ec.getClass()!=CurrentIntegrator.class&&ec.getClass()!=VoltageIntegrator.class
            &&ec.getClass()!=ACVoltageSource.class&&ec.getClass()!=ACCurrentSource.class){
          continue;
        }
      }
      if (ec.originalValue==ec.twoStepValue){
        //ec.isConstant=true;  #commented out dec 21 08
        //System.out.println("=");
        //continue;  //##$$removed dec 21 08 testing
      }
      double delta0;
      if (ec.getClass()==Capacitor.class||ec.getClass()==CurrentIntegrator.class
          ||ec.getClass()==ACCurrentSource.class){
        delta0=myParentCircuitEnginePanel.capacitorError;
      }else if (ec.getClass()==Inductor.class||ec.getClass()==ACVoltageSource.class){
        delta0=myParentCircuitEnginePanel.inductanceError;//error allowed in magnetic flux
      }else{//voltage integrator
        delta0=myParentCircuitEnginePanel.inductanceError;//error allowed in magnetic flux
      }
      //double delta0=Math.max(myParentCircuitEnginePanel.capacitorError*h*Math.abs(h*ec.originalCurrent),
       //                      myParentCircuitEnginePanel.inductanceError);
     // *((//Math.abs(ec.twoStepValue)+
       //   Math.abs(h*ec.originalCurrent)))
     // +myParentCircuitEnginePanel.minimumAbsoluteError
   //   ;//was 10^-30
      if (ec.getClass()==Capacitor.class||ec.getClass()==CurrentIntegrator.class){
        //delta0=Math.min(delta0,0.000001);
      }
      ec.delta1=ec.twoStepValue-ec.oneStepValue;
      if (ec.delta1!=0){//##added Dec 31 2008
        minh=Math.min(minh,
                   0.9*h*Math.pow(//.9
                                delta0
                                /Math.abs(ec.delta1)
                              //Math.abs(0.000000000000000001/(ec.oneStepValue-ec.twoStepValue))
                              ,0.25));
      }
      if (minh<h){
        rk(Math.min(0.5*minh, h),h);
        //System.out.println("recursive rk");
        return;
      }
      //System.out.println(minh);
    }
    for (ElectricComponent ec:myParentCircuitEnginePanel.myCircuit.electricComponents){
      if ((ec.getClass()!=Capacitor.class&&ec.getClass()!=Inductor.class)||ec.primaryValue==null||ec.isConstant){
        if (ec.getClass()!=CurrentIntegrator.class&&ec.getClass()!=VoltageIntegrator.class){
          continue;
        }
      }
      ec.secondaryValue=ec.twoStepValue+ec.delta1/15;
      //System.out.println("delta1: "+ec.delta1);
      //System.out.println("twoStepValue: "+ec.twoStepValue);
      //System.out.println("oneStepValue: "+ec.oneStepValue);
    }
    //rk2(minh,false);
    t+=h;
    this.stepSize=h;
  }
  /**
   * Draws the panel showing all the components. Draws grid points, components,
   * labels. Grid points are only shown near mouse cursor. Components are
   * sometimes drawn in a special color.
   */
  public void paintComponent(Graphics g) {
    super.paintComponent(g);
    drawGridPoints(g);
    //updateCircuit();

    // now we know I, ie k1
    
    drawComponents(g);
    
    for (ElectricComponent ec : myParentCircuitEnginePanel.myCircuit.electricComponents) {
      if (ec.getClass() == Capacitor.class && ec.primaryValue != null) {
        //System.out.println(ec.secondaryValue);
      }
    }
    if (myParentCircuitEnginePanel.getIsDrawing()) {
      drawTempComponent(g);
    }
    Iterator i;
    i = myParentCircuitEnginePanel.myCircuit.electricComponents.iterator();
    // System.out.println(tiePoints.size());
    // boolean
    // isClose=movingState!=MOVING&drawingState!=DRAWING&isCloseToComponent(mouseLocation)&!isClose(new
    // TiePoint(new Point(mouseLocation)), INNER_RADIUS);

  }

  public void drawGridPoints(final Graphics g) {
    for (TiePoint t : myParentCircuitEnginePanel.myGridPoints.values()) {
      if (t.myConnectedTo.size() >= 3) {
        g.setColor(Color.black);
        g.fillOval(t.getX() - 3, t.getY() - 3, 6, 6);
      }
    }
    g.setColor(Color.LIGHT_GRAY);
    if (myParentCircuitEnginePanel.getMouseEntered()) {
      int x1 = CircuitEnginePanel.x1;
      double deltax = CircuitEnginePanel.deltax;
      int y1 = CircuitEnginePanel.y1;
      double deltay = CircuitEnginePanel.deltay;
      int numx = CircuitEnginePanel.numx;
      int numy = CircuitEnginePanel.numy;
      for (int x = x1; x <= x1 + deltax * (numx - 1); x += deltax) {
        for (int y = y1; y <= y1 + deltay * (numy - 1); y += deltay) {
          g.fillOval(x - 1, y - 1, 3, 3);
        }
      }
      for (TiePoint t : myParentCircuitEnginePanel.myGridPoints.values()) {
        g.setColor(Color.black);
        g.fillOval(t.getX() - 3, t.getY() - 3, 6, 6);
      }
      g.setColor(Color.LIGHT_GRAY);
      if (!myParentCircuitEnginePanel.getIsDrawing() &&
          !myParentCircuitEnginePanel.getIsMovingPoint() &&
          !myParentCircuitEnginePanel.getIsMovingComponent()) {
        Point point =
            new Point(myParentCircuitEnginePanel.nearestX(), myParentCircuitEnginePanel
                .nearestY());
        if (myParentCircuitEnginePanel.myGridPoints.containsKey(point) &&
            myParentCircuitEnginePanel.myNearestTiePoint == myParentCircuitEnginePanel.myGridPoints
                .get(point)) {
          g.setColor(Color.blue);
        }
        g.fillOval(myParentCircuitEnginePanel.nearestX() - 4, myParentCircuitEnginePanel
            .nearestY() - 4, 8, 8);
      }
    }
  }

  public void drawComponents(final Graphics g) {
    // System.out.println();
    // System.out.println(myParentCircuitEnginePanel.myGridPoints.values().size());
    g.setColor(Color.black);
    g.drawString("Time: "+this.t+ "s",10,10);
    g.drawString("Step size: "+this.stepSize+"s",10,25);
    for (ElectricComponent e : myParentCircuitEnginePanel.myCircuit.electricComponents) {
      // System.out.println(e.head);
      // System.out.println(e.tail);
      if (!myParentCircuitEnginePanel.getIsDrawing() &&
          //!myParentCircuitEnginePanel.getIsMovingComponent()&&
          !myParentCircuitEnginePanel.getIsMovingPoint()&&!myParentCircuitEnginePanel.nearTiePoint()
          &&myParentCircuitEnginePanel.nearSwitch()&&e== myParentCircuitEnginePanel.myNearestSwitch){
        g.setColor(Color.green);
        e.draw(g);
      }else if (e == myParentCircuitEnginePanel.myNearestComponent) {
        g.setColor(Color.blue);
        e.draw(g);
        g.fillOval(myParentCircuitEnginePanel.myNearestTiePoint.getX() - 4,
                   myParentCircuitEnginePanel.myNearestTiePoint.getY() - 4, 8, 8);
        if (e.getClass()==Ammeter.class||e.getClass()==CurrentIntegrator.class
            ||e.getClass() == VoltageIntegrator.class||e.getClass() == Voltmeter.class
            ||e.getClass() == Ohmmeter.class||e.getClass() == CapacitanceMeter.class
            ||e.getClass() == InductanceMeter.class){
          g.drawString(e.fullValue,10,40);
        }
      } else {
        g.setColor(Color.black);
        e.draw(g);
      }
      int x1 = (int) e.head.getX();
      int y1 = (int) e.head.getY();
      int x2 = (int) e.tail.getX();
      int y2 = (int) e.tail.getY();
      if (e.getClass() == Ammeter.class) {
        //System.out.println(((Ammeter)e).ampString);
        g.drawString(((Ammeter)e).ampString, (int) ((x1 + x2) * 0.5 + 5) -
                                                       Meter.METER_SIZE,
                     (int) (0.5 * (y1 + y2)));
      }
      else if (e.getClass() == CurrentIntegrator.class) {
        g.drawString(((CurrentIntegrator)e).chargeString, (int) ((x1 + x2) * 0.5 + 5) -
                                                       Meter.METER_SIZE,
                     (int) (0.5 * (y1 + y2)));
      }
      else if (e.getClass() == VoltageIntegrator.class) {
        g.drawString(((VoltageIntegrator)e).magneticFluxString, (int) ((x1 + x2) * 0.5 + 4) -//changed 5 to 4 on Dec 25 2008
                                                       Meter.METER_SIZE,
                     (int) (0.5 * (y1 + y2)));
      }
      else if (e.getClass() == Voltmeter.class) {
        g.drawString(((Voltmeter)e).voltageString, (int) ((x1 + x2) * 0.5 + 5) -
                                                 Meter.METER_SIZE, (int) (0.5 * (y1 + y2)));
      }
      else if (e.getClass() == Ohmmeter.class) {
        g.drawString(((Ohmmeter)e).ohmString, (int) ((x1 + x2) * 0.5 + 5) -
                                                    Meter.METER_SIZE, (int) (0.5 * (y1 + y2)));
      }
      else if (e.getClass() == CapacitanceMeter.class) {
        g.drawString(((CapacitanceMeter)e).capacitanceString, (int) ((x1 + x2) * 0.5 + 5) -
                                                 Meter.METER_SIZE, (int) (0.5 * (y1 + y2)));
      }
      else if (e.getClass() == InductanceMeter.class) {
        g.drawString(((InductanceMeter)e).inductanceString, (int) ((x1 + x2) * 0.5 + 5) -
                                                 Meter.METER_SIZE, (int) (0.5 * (y1 + y2)));
      }
    }
  }

  public void drawTempComponent(final Graphics g) {
    g.setColor(Color.GRAY);
    myParentCircuitEnginePanel.myTempComponent.draw(g);
  }
}
